import React from 'react';
import { Meta } from '@storybook/react';
import { graphQueryResultSlice, querybuilderSlice, schemaSlice, visualizationSlice } from '../../../data-access/store';
import { configureStore } from '@reduxjs/toolkit';
import { Provider } from 'react-redux';
import {
  big2ndChamberQueryResult,
  smallFlightsQueryResults,
  mockLargeQueryResults,
  recommendationPersonActedInMovieQueryResultPayload,
  slackReactionToThreadedMessageQueryResultPayload,
} from '../../../mock-data';
import { NodeLinkComponent } from './nodelinkvis';

const Mockstore = configureStore({
  reducer: {
    schema: schemaSlice.reducer,
    graphQueryResult: graphQueryResultSlice.reducer,
    visualize: visualizationSlice.reducer,
    querybuilder: querybuilderSlice.reducer,
  },
});

const Component: Meta<typeof NodeLinkComponent.component> = {
  title: 'Visualizations/NodeLinkVis',
  component: NodeLinkComponent.component,
  decorators: [
    (story) => (
      <Provider store={Mockstore}>
        <div
          style={{
            width: '100%',
            height: '100vh',
          }}
        >
          {story()}
        </div>
      </Provider>
    ),
  ],
};

export const TestWithData = {
  args: {
    data: {
      nodes: [
        { id: 'agent/007', attributes: { name: 'Daniel Craig' } },
        { id: 'villain', attributes: { name: 'Le Chiffre' } },
      ],
      edges: [{ id: 'escape/escape', from: 'agent/007', to: 'villain', attributes: { name: 'Escape' } }],
    },
    ml: {},
    configuration: NodeLinkComponent.configuration,
    dispatch: () => {},
  },
};

export const TestWithDoubleArchData = {
  args: {
    data: {
      nodes: [
        { id: 'agent/007', attributes: { name: 'Daniel Craig' } },
        { id: 'villain', attributes: { name: 'Le Chiffre' } },
      ],
      edges: [
        { id: 'escape/escape', from: 'agent/007', to: 'villain', attributes: { name: 'Escape' } },
        { id: 'escape/escape', to: 'agent/007', from: 'villain', attributes: { name: 'Escape' } },
      ],
    },
    ml: {},
    configuration: NodeLinkComponent.configuration,
    dispatch: () => {},
  },
};

export const TestWithNoData = {
  args: {
    data: {
      nodes: [],
      edges: [],
    },
    ml: {},
    configuration: NodeLinkComponent.configuration,
    dispatch: () => {},
  },
};

export const TestWithBig2ndChamber = {
  args: {
    data: big2ndChamberQueryResult,
    ml: {},
    configuration: NodeLinkComponent.configuration,
    dispatch: () => {},
  },
};

export const TestWithSmallFlights = {
  args: {
    data: smallFlightsQueryResults,
    ml: {},
    configuration: NodeLinkComponent.configuration,
    dispatch: () => {},
  },
};

export const TestWithLargeQueryResult = {
  args: {
    data: mockLargeQueryResults,
    ml: {},
    configuration: NodeLinkComponent.configuration,
    dispatch: () => {},
  },
};

export const TestWithRecommendationPersonActedInMovieQueryResult = {
  args: {
    data: recommendationPersonActedInMovieQueryResultPayload.result.payload,
    ml: {},
    configuration: NodeLinkComponent.configuration,
    dispatch: () => {},
  },
};

export const TestWithSlackReactionToThreadedMessageQueryResult = {
  args: {
    data: slackReactionToThreadedMessageQueryResultPayload.result.payload,
    ml: {},
    configuration: NodeLinkComponent.configuration,
    dispatch: () => {},
  },
};

export default Component;
